package infovis.chartgenerators;
import java.awt.geom.*;

public class ShapeGenerator {
	
	int points=64;
	
	public ShapeGenerator(){		
	}
	
	public float[] Box(float w, float h){
		float[] shape=new float[points*2];

		double u=2.0*w+h*2.0;
		int pw=(int)(w*points/u);
		int ph=(points-(pw*2))/2;
		float pdh=h/(float)(ph);
		float pdw=w/(float)(pw);
		
		for(int i=0; i<ph; i++){
			shape[Py(i)]=-h*0.5f+i*pdh;
			shape[Px(i)]=w*0.5f;
			
			shape[Py(i+ph+pw)]=h*0.5f-i*pdh;
			shape[Px(i+ph+pw)]=-w*0.5f;
						
		}
		
		for(int i=0; i<pw; i++){
			shape[Px(i+ph)]=w*0.5f-i*pdw;
			shape[Py(i+ph)]=h*0.5f;
			
			shape[Px(i+ph+ph+pw)]=-w*0.5f+i*pdw;				
			shape[Py(i+ph+ph+pw)]=-h*0.5f;
		}
				
		shape=reorder(shape, 0.0f, 0.0f);
		
		return shape;
	}
	
	public float[] Circle(float radius){
		float[] shape=new float[points*2];
		
		double a=Math.PI*2.0f/(float)(points);
		double ang=0.0f;
		
		for(int i=0; i<points; i++) {
			shape[Px(i)]=(float)Math.cos(ang)*radius;
			shape[Py(i)]=(float)Math.sin(ang)*radius;
			ang+=a;
		}
		
		return shape;
	}
	
	
	public float[] Pie(float r, float v, float ang){
		float[] shape=new float[points*2];
		
		ang=(float)(2.0f*Math.PI*(ang+0.25f));
		
		double ubogen=2.0*Math.PI*r*v;
		double utotal=ubogen+2.0f*r;
		
		int pb=(int)((ubogen/utotal)*(double)(points));
		int pl=(int)((r/utotal)*(double)(points));
		
		double vx;
		double vy;
		
		vx=Math.cos(ang)*r;
		vy=Math.sin(ang)*r;
	
		vx=vx/(float)pl;
		vy=vy/(float)pl;
		
		for(int i=0; i<pl; i++) {
			shape[Px(i)]=(float)vx*i;
			shape[Py(i)]=(float)vy*i;
		}
		
		double a=(2.0f*Math.PI*v)/(float)(pb-1);
		double angl=ang;
		
		for(int i=pl; i<pl+pb; i++){
			shape[Px(i)]=(float)Math.cos(angl)*r;
			shape[Py(i)]=(float)Math.sin(angl)*r;
			angl+=a;
		}
		
		float rest=points-pl-pb;
		
		vx=Math.cos((2.0f*Math.PI*v)+ang)*r;
		vy=Math.sin((2.0f*Math.PI*v)+ang)*r;
		
		vx=vx/rest;
		vy=vy/rest;
		
		for(int i=0; i<(int)rest+1; i++){
			shape[Px(points-i)]=(float)vx*i;
			shape[Py(points-i)]=(float)vy*i;
		}
		
		float MPx=(float)Math.cos(2.0f*Math.PI*(v*0.5f)+ang)*r*0.5f;
		float MPy=(float)Math.sin(2.0f*Math.PI*(v*0.5f)+ang)*r*0.5f;
		
		//System.out.println(MPx+" "+MPy);
		
		//debug(shape);
		
		shape=reorder(shape, MPx, MPy);
		
		for(int i=0; i<points; i++){
			shape[Px(i)]-=MPx;
			shape[Py(i)]-=MPy;
		}
		
		return shape;
	}
	
	public float[] reorder(float[] s, float MPx, float MPy){
		float[] shape=new float[points*2];
		
		int c=0;
			
		while(!(new Line2D.Float(MPx,MPy,MPx+1.0f,MPy)).intersectsLine(s[Px(c)],s[Py(c)],s[Px(c+1)],s[Py(c+1)])){ 
			c++; 
		}
		
		for(int i=0; i<points; i++){
			shape[Px(i)]=s[Px(i+c)];
			shape[Py(i)]=s[Py(i+c)];
		}
		
		return shape;
	}
	
	public int Px(int i) {
			return (i%points)*2;
		}

	public int Py(int i) { 
			return (i%points)*2+1;
	}
	
	public void debug(float[] s){
		
		for(int i=0; i<s.length/2; i++)
		System.out.println(i+" x"+s[Px(i)]+" y"+s[Py(i)]);
	}
	
}
